/* dlg_config.h.  Generated automatically by configure.  */
/*
 * The configure script expands this as a set of definitions
 */

#define DLG_CURSES_WACS_ARRAY _nc_wacs
#define DLG_CURSES_WACS_SYMBOLS 1
#define DLG_DIALOG_PATCHDATE 20260107
#define DLG_DIALOG_VERSION "1.3"
#define DLG_ENABLE_NLS 1
#define DLG_GCC_NORETURN __attribute__((noreturn))
#define DLG_GCC_PRINTF 1
#define GCC_PRINTFLIKE(fmt,var) __attribute__((format(printf,fmt,var)))
#define DLG_GCC_SCANF 1
#define GCC_SCANFLIKE(fmt,var) __attribute__((format(scanf,fmt,var)))
#define DLG_GCC_UNUSED __attribute__((unused))
#define DLG_HAVE_BTOWC 1
#define DLG_HAVE_COLOR 1
#define DLG_HAVE_DCGETTEXT 1
#define DLG_HAVE_DIRENT_H 1
#define DLG_HAVE_DLG_FORMBOX 1
#define DLG_HAVE_DLG_GAUGE 1
#define DLG_HAVE_DLG_MIXEDFORM 1
#define DLG_HAVE_DLG_TAILBOX 1
#define DLG_HAVE_DLG_TRACE 1
#define DLG_HAVE_EXIT_CURSES 1
#define DLG_HAVE_FLUSHINP 1
#define DLG_HAVE_FSEEKO 1
#define DLG_HAVE_GETATTRS 1
#define DLG_HAVE_GETBEGX 1
#define DLG_HAVE_GETBEGY 1
#define DLG_HAVE_GETBEGYX 1
#define DLG_HAVE_GETCURX 1
#define DLG_HAVE_GETCURY 1
#define DLG_HAVE_GETMAXX 1
#define DLG_HAVE_GETMAXY 1
#define DLG_HAVE_GETMAXYX 1
#define DLG_HAVE_GETPARX 1
#define DLG_HAVE_GETPARY 1
#define DLG_HAVE_GETPARYX 1
#define DLG_HAVE_GETTEXT 1
#define DLG_HAVE_ICONV 1
#define DLG_HAVE_INTTYPES_H 1
#define DLG_HAVE_LANGINFO_CODESET 1
#define DLG_HAVE_LC_MESSAGES 1
#define DLG_HAVE_LIBINTL_H 1
#define DLG_HAVE_MATH_FUNCS 1
#define DLG_HAVE_MBLEN 1
#define DLG_HAVE_MBRLEN 1
#define DLG_HAVE_MBRTOWC 1
#define DLG_HAVE_MBSTATE_T 1
#define DLG_HAVE_MBTOWC 1
#define DLG_HAVE_MEMORY_H 1
#define DLG_HAVE_MIXEDGAUGE 1
#define DLG_HAVE_MKTIME 1
#define DLG_HAVE_NCURSES_H 1
#define DLG_HAVE_RC_FILE 1
#define DLG_HAVE_RC_FILE2 1
#define DLG_HAVE_SEARCH_H 1
#define DLG_HAVE_SETLOCALE 1
#define DLG_HAVE_SIZECHANGE 1
#define DLG_HAVE_STDINT_H 1
#define DLG_HAVE_STDLIB_H 1
#define DLG_HAVE_STRCASECMP 1
#define DLG_HAVE_STRFTIME 1
#define DLG_HAVE_STRINGS_H 1
#define DLG_HAVE_STRING_H 1
#define DLG_HAVE_SYS_IOCTL_H 1
#define DLG_HAVE_SYS_PARAM_H 1
#define DLG_HAVE_SYS_STAT_H 1
#define DLG_HAVE_SYS_TYPES_H 1
#define DLG_HAVE_SYS_WAIT_H 1
#define DLG_HAVE_TERMIOS_H 1
#define DLG_HAVE_TERM_H 1
#define DLG_HAVE_TSEARCH 1
#define DLG_HAVE_TYPE_CHTYPE 1
#define DLG_HAVE_UNCTRL_H 1
#define DLG_HAVE_UNISTD_H 1
#define DLG_HAVE_USE_DEFAULT_COLORS 1
#define DLG_HAVE_WAITPID 1
#define DLG_HAVE_WCHGAT 1
#define DLG_HAVE_WCSRTOMBS 1
#define DLG_HAVE_WCSTOMBS 1
#define DLG_HAVE_WCTOB 1
#define DLG_HAVE_WCTOMB 1
#define DLG_HAVE_WCURSYNCUP 1
#define DLG_HAVE_WGETPARENT 1
#define DLG_HAVE_WGET_WCH 1
#define DLG_HAVE_WHIPTAIL 1
#define DLG_HAVE_WSYNCUP 1
#define DLG_HAVE_XDIALOG 1
#define DLG_HAVE_XDIALOG2 1
#define DLG_ICONV_CONST const
#define DLG_MIXEDCASE_FILENAMES 1
#define DLG_NCURSES 1
#define DLG_NEED_WCHAR_H 1
#define DLG_NLS_TEXTDOMAIN "dialog"
#define DLG_PACKAGE "dialog"
#define DLG_RETSIGTYPE void
#define DLG_STDC_HEADERS 1
#define DLG_SYSTEM_NAME "openbsd7.9"
#define DLG_TIME_WITH_SYS_TIME 1
#define DLG_TYPE_CHTYPE_IS_SCALAR 1
#define DLG_USE_WIDE_CURSES 1
#define DLG_WIDEC_CURSES 1
